@@ -48,7 +48,6 @@ pub struct Editor<XMSG> {
     #[allow(unused)]
     page_size: usize,
     /// for undo and redo
-    #[allow(unused)]
     recorded: Recorded,
     measurements: Option<Measurements>,
     average_update_time: Option<f64>,
@@ -149,7 +148,12 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
             }
             Msg::TextareaKeydown(ke) => {
                 let is_ctrl = ke.ctrl_key();
-                log::trace!("text area key down... {}", is_ctrl);
+                let is_shift = ke.shift_key();
+                log::trace!(
+                    "text area key down... ctrl: {} ,shift: {}",
+                    is_ctrl,
+                    is_shift
+                );
                 // don't process key presses when
                 // CTRL key is pressed.
                 let key = ke.key();
@@ -158,25 +162,35 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                     let c = key.chars().next().expect("must be only 1 chr");
                     match c {
                         'c' if is_ctrl => {
-                            let ret = self.command_copy();
-                            log::trace!("copy works: {}", ret);
-                        }
-                        'x' if is_ctrl => {
-                            let ret = self.command_cut();
-                            log::trace!("cut works: {}", ret);
+                            self.command_copy();
+                            Effects::none()
                         }
+                        'x' if is_ctrl => self.command_cut(),
                         'v' if is_ctrl => {
                             log::trace!("pasting is handled");
                             self.clear_hidden_textarea();
+                            let extern_msgs = self.emit_on_change_listeners();
+                            Effects::with_external(extern_msgs).measure()
+                        }
+                        'z' | 'Z' if is_ctrl => {
+                            if is_shift {
+                                self.recorded.redo(&mut self.text_buffer);
+                            } else {
+                                self.recorded.undo(&mut self.text_buffer);
+                            }
+                            let extern_msgs = self.emit_on_change_listeners();
+                            Effects::with_external(extern_msgs).measure()
                         }
                         _ => {
                             self.command_insert_char(c);
                             self.clear_hidden_textarea();
+                            let extern_msgs = self.emit_on_change_listeners();
+                            Effects::with_external(extern_msgs).measure()
                         }
                     }
+                } else {
+                    Effects::none()
                 }
-                let extern_msgs = self.emit_on_change_listeners();
-                Effects::with_external(extern_msgs).measure()
             }
             Msg::Mouseup(client_x, client_y) => {
                 let cursor = self.client_to_cursor(client_x, client_y);
@@ -197,14 +211,18 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 Effects::none().measure()
             }
             Msg::Mousedown(client_x, client_y) => {
-                let cursor = self.client_to_cursor(client_x, client_y);
-                self.is_selecting = true;
-                self.selection_start = Some(cursor);
-                self.command_set_position(cursor.x, cursor.y);
+                if self.in_bounds(client_x as f32, client_y as f32) {
+                    let cursor = self.client_to_cursor(client_x, client_y);
+                    self.is_selecting = true;
+                    self.selection_start = Some(cursor);
+                    self.command_set_position(cursor.x, cursor.y);
+                }
                 Effects::none().measure()
             }
             Msg::Mousemove(client_x, client_y) => {
-                if self.is_selecting {
+                if self.is_selecting
+                    && self.in_bounds(client_x as f32, client_y as f32)
+                {
                     let cursor = self.client_to_cursor(client_x, client_y);
                     self.selection_end = Some(cursor);
 
@@ -339,62 +357,68 @@ impl<XMSG> Editor<XMSG> {
         self
     }
 
-    fn command_insert_char(&mut self, ch: char) {
+    fn command_insert_char(&mut self, ch: char) -> Effects<Msg, XMSG> {
         self.text_buffer.command_insert_char(ch);
+        self.recorded.insert(ch);
         self.text_buffer.rehighlight();
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
     }
 
-    pub fn command_replace_char(&mut self, ch: char) {
+    pub fn command_replace_char(&mut self, ch: char) -> Effects<Msg, XMSG> {
         self.text_buffer.command_replace_char(ch);
         self.text_buffer.rehighlight();
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
     }
 
-    fn command_break_line(&mut self) {
+    fn command_break_line(&mut self) -> Effects<Msg, XMSG> {
         self.text_buffer.command_break_line();
         self.text_buffer.rehighlight();
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
     }
 
-    fn command_insert_text(&mut self, text: &str) {
+    fn command_insert_text(&mut self, text: &str) -> Effects<Msg, XMSG> {
         self.text_buffer.command_insert_text(text);
         self.text_buffer.rehighlight();
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
     }
 
     pub fn command_set_position(&mut self, cursor_x: i32, cursor_y: i32) {
-        if self.text_buffer.in_bounds(Point2::new(cursor_x, cursor_y)) {
-            self.text_buffer
-                .set_position(cursor_x as usize, cursor_y as usize);
-        }
+        self.text_buffer
+            .set_position(cursor_x as usize, cursor_y as usize);
     }
 
     fn command_set_selection(&mut self, start: Point2<i32>, end: Point2<i32>) {
-        if self.text_buffer.in_bounds(start) && self.text_buffer.in_bounds(end)
-        {
-            let start = Point2::new(start.x as usize, start.y as usize);
-            let end = Point2::new(end.x as usize, end.y as usize);
-            self.text_buffer.set_selection(start, end);
-        }
+        let start = Point2::new(start.x as usize, start.y as usize);
+        let end = Point2::new(end.x as usize, end.y as usize);
+        self.text_buffer.set_selection(start, end);
     }
 
     /// calls on 2 ways to copy
     /// either 1 should work
     /// returns true if it succeded
-    fn command_copy(&self) -> bool {
+    fn command_copy(&self) {
         if self.copy_to_clipboard() {
-            true
+            // do nothing
         } else {
-            self.textarea_exec_copy()
+            self.textarea_exec_copy();
         }
     }
 
     /// try exec_cut, try cut to clipboard if the first fails
     /// This shouldn't execute both since cut is destructive.
     /// Returns true if it succeded
-    fn command_cut(&mut self) -> bool {
+    fn command_cut(&mut self) -> Effects<Msg, XMSG> {
         if self.cut_to_clipboard() {
-            true
+            // nothing
         } else {
-            self.textarea_exec_cut()
+            self.textarea_exec_cut();
         }
+        let extern_msgs = self.emit_on_change_listeners();
+        Effects::with_external(extern_msgs).measure()
     }
 
     /// set the content of the textarea to selection
@@ -512,10 +536,12 @@ impl<XMSG> Editor<XMSG> {
                 self.text_buffer.command_break_line();
             }
             "Backspace" => {
-                self.text_buffer.command_delete_back();
+                let c = self.text_buffer.command_delete_back();
+                self.recorded.delete(c);
             }
             "Delete" => {
-                self.text_buffer.command_delete_forward();
+                let c = self.text_buffer.command_delete_forward();
+                self.recorded.delete_forward(c);
             }
             "ArrowUp" => {
                 self.text_buffer.move_up();
@@ -587,13 +613,34 @@ impl<XMSG> Editor<XMSG> {
         extern_msgs
     }
 
-    pub fn editor_offset(&self) -> Option<Point2<f32>> {
+    pub fn bounding_rect(&self) -> Option<(Point2<f32>, Point2<f32>)> {
         if let Some(ref editor_element) = self.editor_element {
-            log::trace!("calculating editor offset..");
             let rect = editor_element.get_bounding_client_rect();
             let editor_x = rect.x().round() as f32;
             let editor_y = rect.y().round() as f32;
-            Some(Point2::new(editor_x, editor_y))
+            let bottom = rect.bottom().round() as f32;
+            let right = rect.right().round() as f32;
+            Some((Point2::new(editor_x, editor_y), Point2::new(right, bottom)))
+        } else {
+            None
+        }
+    }
+
+    /// check if this mouse client x and y is inside the editor bounds
+    pub fn in_bounds(&self, client_x: f32, client_y: f32) -> bool {
+        if let Some((start, end)) = self.bounding_rect() {
+            client_x >= start.x
+                && client_x <= end.x
+                && client_y >= start.y
+                && client_y <= end.y
+        } else {
+            false
+        }
+    }
+
+    pub fn editor_offset(&self) -> Option<Point2<f32>> {
+        if let Some((start, _end)) = self.bounding_rect() {
+            Some(start)
         } else {
             None
         }
@@ -700,7 +747,7 @@ impl<XMSG> Editor<XMSG> {
         )
     }
 
-    fn view_status_line<Msg>(&self) -> Node<Msg> {
+    pub fn view_status_line<Msg>(&self) -> Node<Msg> {
         let class_ns = |class_names| {
             attributes::class_namespaced(COMPONENT_NAME, class_names)
         };
@@ -724,10 +771,11 @@ impl<XMSG> Editor<XMSG> {
                 },
             ],
             [
-                text!("line: {}, col: {}  |", cursor.y + 1, cursor.x + 1),
+                text!("line: {}, col: {} ", cursor.y + 1, cursor.x + 1),
                 if let Some(measurements) = &self.measurements {
                     text!(
-                        "patches: {} | nodes: {} | view time: {}ms | patch time: {}ms | update time: {}ms",
+                        "| msgs: {} | patches: {} | nodes: {} | view time: {}ms | patch time: {}ms | update time: {}ms",
+                        measurements.msg_count,
                         measurements.total_patches,
                         measurements.view_node_count,
                         measurements.build_view_took.round(),
@@ -742,6 +790,7 @@ impl<XMSG> Editor<XMSG> {
                 } else {
                     comment("")
                 },
+                text!("| version:{}", env!("CARGO_PKG_VERSION")),
             ],
         )
     }
