@@ -82,6 +82,11 @@ impl TextBuffer {
         self.selection_end = Some(end);
     }
 
+    pub fn clear_selection(&mut self) {
+        self.selection_start = None;
+        self.selection_end = None;
+    }
+
     /// return the min and max selection bound
     pub fn normalize_selection(
         &self,
@@ -236,12 +241,6 @@ impl TextBuffer {
         }
     }
 
-    /// check if x and y is in the bounds of the texteditor longest column
-    pub fn in_bounds(&self, point: Point2<i32>) -> bool {
-        let bound = self.bounds();
-        point.x >= 0 && point.y >= 0 && point.x <= bound.x && point.y <= bound.y
-    }
-
     /// calculate the bounds of the text_buffer
     pub fn bounds(&self) -> Point2<i32> {
         let total_lines = self.lines.len() as i32;
@@ -733,18 +732,20 @@ impl TextBuffer {
     }
 
     /// delete character at this position
-    pub(crate) fn delete_char(&mut self, x: usize, y: usize) {
+    pub(crate) fn delete_char(&mut self, x: usize, y: usize) -> Option<char> {
         if let Some(line) = self.lines.get_mut(y) {
             if let Some((range_index, col)) =
                 line.calc_range_cell_index_position(x)
             {
                 if let Some(range) = line.ranges.get_mut(range_index) {
                     if range.cells.get(col).is_some() {
-                        range.cells.remove(col);
+                        let cell = range.cells.remove(col);
+                        return Some(cell.ch);
                     }
                 }
             }
         }
+        None
     }
 
     fn ensure_cell_exist(&mut self, x: usize, y: usize) {
@@ -796,6 +797,11 @@ impl TextBuffer {
         self.move_x(width);
     }
 
+    /// insert the character but don't move to the right
+    pub(crate) fn command_insert_forward_char(&mut self, ch: char) {
+        self.insert_char(self.cursor.x, self.cursor.y, ch);
+    }
+
     pub(crate) fn command_replace_char(&mut self, ch: char) {
         self.replace_char(self.cursor.x, self.cursor.y, ch);
     }
@@ -846,14 +852,33 @@ impl TextBuffer {
         self.move_left_start();
         self.move_down();
     }
-    pub(crate) fn command_delete_back(&mut self) {
+    pub(crate) fn command_delete_back(&mut self) -> Option<char> {
         if self.cursor.x > 0 {
-            self.delete_char(self.cursor.x.saturating_sub(1), self.cursor.y);
+            let c = self
+                .delete_char(self.cursor.x.saturating_sub(1), self.cursor.y);
             self.move_left();
+            c
+        } else {
+            None
         }
     }
-    pub(crate) fn command_delete_forward(&mut self) {
-        self.delete_char(self.cursor.x, self.cursor.y);
+    pub(crate) fn command_delete_forward(&mut self) -> Option<char> {
+        let c = self.delete_char(self.cursor.x, self.cursor.y);
+        self.calculate_focused_cell();
+        c
+    }
+    pub(crate) fn command_delete_selected_forward(&mut self) -> Option<String> {
+        if let Some((start, end)) = self.normalize_selection() {
+            let deleted_text = self.cut_text(start, end);
+            self.move_to(start);
+            Some(deleted_text)
+        } else {
+            None
+        }
+    }
+    pub(crate) fn move_to(&mut self, pos: Point2<usize>) {
+        self.cursor.x = pos.x;
+        self.cursor.y = pos.y;
         self.calculate_focused_cell();
     }
 }
