@@ -76,17 +76,10 @@ impl Line {
         width
     }
 
-    #[allow(unused)]
-    pub(super) fn last_range_cell_index(&self) -> (usize, usize) {
-        let line_ranges_len = self.ranges.len();
-        let last = if line_ranges_len > 0 {
-            line_ranges_len - 1
-        } else {
-            0
-        };
-
+    /// The location of the the next cell to be appended to the last range of this line
+    pub(super) fn range_cell_next(&self) -> (usize, usize) {
         (
-            last,
+            self.ranges.len().saturating_sub(1),
             self.ranges
                 .last()
                 .map(|ranges| ranges.cells.len())
@@ -94,6 +87,17 @@ impl Line {
         )
     }
 
+    /// Return the indexes of the last range and the last cell of the last range of the line
+    pub(super) fn range_cell_tail(&self) -> (usize, usize) {
+        (
+            self.ranges.len().saturating_sub(1),
+            self.ranges
+                .last()
+                .map(|ranges| ranges.cells.len().saturating_sub(1))
+                .unwrap_or(0),
+        )
+    }
+
     pub(super) fn view_line<MSG>(
         &self,
         text_buffer: &TextBuffer,
@@ -195,6 +199,70 @@ impl Line {
         }
     }
 
+    pub(super) fn recalc_width(&mut self) {
+        self.width = self.ranges.iter().map(|range| range.width).sum();
+    }
+
+    /// delete from x to last
+    pub(super) fn delete_cells_to_end(&mut self, start_x: usize) {
+        if let Some((range_index, cell_index)) =
+            self.calc_range_cell_index_position(start_x)
+        {
+            log::trace!("delete cells to end, starting from: {}", start_x);
+            log::trace!(
+                "range_index: {}, cell_index: {}",
+                range_index,
+                cell_index
+            );
+            self.ranges[range_index].delete_cells_to_end(cell_index);
+            // drain all ranges beyond `range_index`
+            self.ranges.drain(range_index + 1..);
+            self.recalc_width();
+        }
+    }
+
+    /// delete from the 0 to end_x
+    pub(super) fn delete_cells_from_start(&mut self, end_x: usize) {
+        if let Some((range_index, cell_index)) =
+            self.calc_range_cell_index_position(end_x)
+        {
+            log::trace!("deleting cells from start to {}", end_x);
+            log::trace!(
+                "range_index: {}, cell_index: {}",
+                range_index,
+                cell_index
+            );
+            self.ranges[range_index].delete_cells_from_start(cell_index);
+            // delete ranges from 0 to before this end_x location
+            self.ranges.drain(0..range_index);
+            self.recalc_width();
+        }
+    }
+
+    pub(super) fn delete_cells(&mut self, start_x: usize, end_x: usize) {
+        println!("deleting cells from {} to {}", start_x, end_x);
+        let start = self.calc_range_cell_index_position(start_x);
+
+        // if None, it is because it beyond the cell index of this line, so we use the last_range
+        // cell
+        let end = self
+            .calc_range_cell_index_position(end_x)
+            .unwrap_or(self.range_cell_tail());
+        dbg!(&start);
+        dbg!(&end);
+        if let Some((start_range, start_cell)) = start {
+            let (end_range, end_cell) = end;
+            if start_range == end_range {
+                self.ranges[start_range].delete_cells(start_cell, end_cell);
+            } else {
+                self.ranges[end_range].delete_cells_from_start(end_cell);
+                self.ranges[start_range].delete_cells_to_end(start_cell);
+            }
+            self.ranges.drain(start_range..end_range);
+            self.recalc_width();
+        }
+    }
+
     pub(super) fn insert_char(
         &mut self,
         range_index: usize,
